@@ -1,7 +1,7 @@
 /*
- * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
- * Copyright(c) 1995-99 Andrew Lister
- * Copyright (c) 1999, 2000, 2001 by the LessTif Developers.
+ * Copyright (c) 1992 Bell Communications Research, Inc. (Bellcore)
+ * Copyright (c) 1995-99 Andrew Lister
+ * Copyright  1999, 2000, 2001, 2002, 2003, 2004 by the LessTif Developers.
  *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
@@ -24,7 +24,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
- * $Id: Matrix.h,v 1.9 2002/08/05 08:04:02 dannybackx Exp $
+ * $Id: Matrix.h,v 1.48 2006/05/15 15:57:04 tobiasoed Exp $
  */
 
 #ifndef _Xbae_Matrix_h
@@ -43,6 +43,11 @@
 extern "C" {
 #endif
 
+#if XmVERSION < 2
+typedef char *XmStringTag;
+#define	XmUNSPECIFIED_PIXEL	((Pixel) (~0))
+#endif
+
 /* get version info */
 extern const char *XbaeGetVersionTxt(void);
 extern int XbaeGetVersionNum(void);
@@ -77,10 +82,13 @@ extern int XbaeGetXmVersionNum(void);
  * columnLabelAlignments Alignments		AlignmentArray	dynamic
  * columnLabelColor	Color			Pixel		dynamic
  * columnLabels		Labels			StringArray	NULL
+ * xmColumnLabels	XmLabels		XmStringTable	NULL
  * columnMaxLengths	ColumnMaxLengths	MaxLengthArray	NULL
  * columnShadowTypes	ShadowTypes		ShadowTypeArray NULL
  * columnUserData	UserDatas		UserDataArray	NULL
  * columnWidths		ColumnWidths		WidthArray	NULL
+ * columnWidthInPixels  ColumnWidthInPixels     Boolean         False
+ * rowHeightInPixels    RowHeightInPixels       Boolean         True
  * columns		Columns			int		0
  * defaultActionCallback Callback               Callback        NULL
  * doubleClickInterval  Interval                int             dynamic
@@ -88,6 +96,8 @@ extern int XbaeGetXmVersionNum(void);
  * enterCellCallback	Callback		Callback	NULL
  * evenRowBackground	Background		Pixel		dynamic
  * fill			Fill			Boolean		False
+ * vertFill             Fill                    Boolean         False
+ * horzFill             Fill                    Boolean         False
  * fixedColumns		FixedColumns		Dimension	0
  * fixedRows		FixedRows		Dimension	0
  * fontList		FontList		FontList	fixed
@@ -135,6 +145,9 @@ extern int XbaeGetXmVersionNum(void);
  * trailingFixedRows	TrailingFixedRows	Dimension	0
  * traverseCellCallback	Callback		Callback	NULL
  * traverseFixedCells	TraverseFixedCells	Boolean		False
+ * underlinedCells	UnderlinedCells		BooleanTable	dynamic
+ * underlinePosition	UnderlinePosition		Position	1
+ * underlineWidth	UnderlineWidth		Dimension	1
  * useXbaeInput		UseXbaeInput		Boolean		True
  * valueChangedCallback       Callback                Callback        NULL
  * verticalScrollBar	VerticalScrollBar	Widget		NULL (get only)
@@ -167,9 +180,6 @@ extern int XbaeGetXmVersionNum(void);
 #ifndef XmNcalcCursorPosition
 #define XmNcalcCursorPosition		"calcCursorPosition"
 #endif
-#ifndef XmNcellBackground
-#define XmNcellBackground		"cellBackground"
-#endif
 #ifndef XmNcellBackgrounds
 #define XmNcellBackgrounds		"cellBackgrounds"
 #endif
@@ -200,9 +210,6 @@ extern int XbaeGetXmVersionNum(void);
 #ifndef XmNcells
 #define XmNcells			"cells"
 #endif
-#ifndef XmNcolor
-#define XmNcolor			"color"
-#endif
 #ifndef XmNcolors
 #define XmNcolors			"colors"
 #endif
@@ -215,6 +222,9 @@ extern int XbaeGetXmVersionNum(void);
 #ifndef XmNcolumnFontBold
 #define XmNcolumnFontBold		"columnFontBold"
 #endif
+#ifndef XmNshowColumnArrows
+#define XmNshowColumnArrows		"showColumnArrows"
+#endif
 #ifndef XmNcolumnLabelAlignments
 #define XmNcolumnLabelAlignments	"columnLabelAlignments"
 #endif
@@ -227,6 +237,15 @@ extern int XbaeGetXmVersionNum(void);
 #ifndef XmNcolumnLabels
 #define XmNcolumnLabels			"columnLabels"
 #endif
+#ifndef XmNxmColumnLabels
+#define XmNxmColumnLabels		"xmColumnLabels"
+#endif
+#ifndef XmNXmColumnLabels       /* For backwards compatibility */
+#define XmNXmColumnLabels		XmNxmColumnLabels
+#endif
+#ifndef XmNxmRowLabels
+#define XmNxmRowLabels		"xmRowLabels"
+#endif
 #ifndef XmNcolumnMaxLengths
 #define XmNcolumnMaxLengths		"columnMaxLengths"
 #endif
@@ -239,6 +258,12 @@ extern int XbaeGetXmVersionNum(void);
 #ifndef XmNcolumnWidths
 #define XmNcolumnWidths			"columnWidths"
 #endif
+#ifndef XmNcolumnWidthInPixels
+#define XmNcolumnWidthInPixels         "columnWidthInPixels"
+#endif
+#ifndef XmNrowHeightInPixels
+#define XmNrowHeightInPixels           "rowHeightInPixels"
+#endif
 #ifndef XmNdrawCellCallback
 #define XmNdrawCellCallback		"drawCellCallback"
 #endif
@@ -251,6 +276,12 @@ extern int XbaeGetXmVersionNum(void);
 #ifndef XmNfill
 #define XmNfill				"fill"
 #endif
+#ifndef XmNvertFill
+#define XmNvertFill			"vertFill"
+#endif
+#ifndef XmNhorzFill
+#define XmNhorzFill			"horzFill"
+#endif
 #ifndef XmNfixedColumns
 #define XmNfixedColumns			"fixedColumns"
 #endif
@@ -347,12 +378,21 @@ extern int XbaeGetXmVersionNum(void);
 #ifndef XmNtextBackground
 #define XmNtextBackground		"textBackground"
 #endif
+#ifndef XmNtextBackgroundIsCell
+#define XmNtextBackgroundIsCell		"textBackgroundIsCell"
+#endif
 #ifndef XmNtextField
 #define XmNtextField			"textField"
 #endif
 #ifndef XmNtopRow
 #define XmNtopRow			"topRow"
 #endif
+#ifndef XmNnonFixedDetachedTop
+#define XmNnonFixedDetachedTop	"nonFixedDetachedTop"
+#endif
+#ifndef XmNnonFixedDetachedLeft
+#define XmNnonFixedDetachedLeft	"nonFixedDetachedLeft"
+#endif
 #ifndef XmNtrailingAttachedBottom
 #define XmNtrailingAttachedBottom	"trailingAttachedBottom"
 #endif
@@ -377,6 +417,15 @@ extern int XbaeGetXmVersionNum(void);
 #ifndef XmNtraverseFixedCells
 #define XmNtraverseFixedCells		"traverseFixedCells"
 #endif
+#ifndef XmNunderlinedCells
+#define XmNunderlinedCells		"underlinedCells"
+#endif
+#ifndef XmNunderlinePosition
+#define XmNunderlinePosition		"underlinePosition"
+#endif
+#ifndef XmNunderlineWidth
+#define XmNunderlineWidth		"underlineWidth"
+#endif
 #ifndef XmNvalueChangedCallback
 #define XmNvalueChangedCallback               "valueChangedCallback"
 #endif
@@ -392,6 +441,9 @@ extern int XbaeGetXmVersionNum(void);
 #ifndef XmNwriteCellCallback
 #define XmNwriteCellCallback		"writeCellCallback"
 #endif
+#ifndef	XmNtrackCellCallback
+#define	XmNtrackCellCallback		"trackCellCallback"
+#endif
 
 
 #ifndef XmCAlignments
@@ -436,9 +488,21 @@ extern int XbaeGetXmVersionNum(void);
 #ifndef XmCColumnWidths
 #define XmCColumnWidths			"ColumnWidths"
 #endif
+#ifndef XmCColumnWidthInPixels
+#define XmCColumnWidthInPixels		"ColumnWidthInPixels"
+#endif
+#ifndef XmCRowHeightInPixels
+#define XmCRowHeightInPixels		"RowHeightInPixels"
+#endif
 #ifndef XmCFill
 #define XmCFill				"Fill"
 #endif
+#ifndef XmCVertFill
+#define XmCVertFill			"VertFill"
+#endif
+#ifndef XmCHorzFill
+#define XmCHorzFill			"HorzFill"
+#endif
 #ifndef XmCFixedColumns
 #define XmCFixedColumns			"FixedColumns"
 #endif
@@ -454,6 +518,9 @@ extern int XbaeGetXmVersionNum(void);
 #ifndef XmCLabels
 #define XmCLabels			"Labels"
 #endif
+#ifndef XmCXmLabels
+#define XmCXmLabels			"XmLabels"
+#endif
 #ifndef XmCLeftColumn
 #define XmCLeftColumn			"LeftColumn"
 #endif
@@ -478,6 +545,9 @@ extern int XbaeGetXmVersionNum(void);
 #ifndef XmCTextBackground
 #define XmCTextBackground		"TextBackground"
 #endif
+#ifndef XmCTextBackgroundIsCell
+#define XmCTextBackgroundIsCell		"TextBackgroundIsCell"
+#endif
 #ifndef XmCTextField
 #define XmCTextField			"TextField"
 #endif
@@ -490,6 +560,12 @@ extern int XbaeGetXmVersionNum(void);
 #ifndef XmCTopRow
 #define XmCTopRow			"TopRow"
 #endif
+#ifndef XmCNonFixedDetachedTop
+#define XmCNonFixedDetachedTop	"NonFixedDetachedTop"
+#endif
+#ifndef XmCNonFixedDetachedLeft
+#define XmCNonFixedDetachedLeft	"NonFixedDetachedLeft"
+#endif
 #ifndef XmCTrailingAttachedBottom
 #define XmCTrailingAttachedBottom	"TrailingAttachedBottom"
 #endif
@@ -502,6 +578,15 @@ extern int XbaeGetXmVersionNum(void);
 #ifndef XmCTrailingFixedRows
 #define XmCTrailingFixedRows		"TrailingFixedRows"
 #endif
+#ifndef XmCUnderlinedCells
+#define XmCUnderlinedCells		"UnderlinedCells"
+#endif
+#ifndef XmCUnderlinePosition
+#define XmCUnderlinePosition		"UnderlinePosition"
+#endif
+#ifndef XmCUnderlineWidth
+#define XmCUnderlineWidth		"UnderlineWidth"
+#endif
 #ifndef XmCUserDatas
 #define XmCUserDatas			"UserDatas"
 #endif
@@ -576,13 +661,53 @@ extern int XbaeGetXmVersionNum(void);
 #define	XmCMultiLineCell		"MultiLineCell"
 #endif
 
+#ifndef	XmNwrapType
+#define	XmNwrapType				"wrapType"
+#endif
+#ifndef	XmCWrapType
+#define	XmCWrapType				"WrapType"
+#endif
+#ifndef	XmRWrapType
+#define	XmRWrapType				"WrapType"
+#endif
+
+#ifndef XmNdesiredHeight
+#define XmNdesiredHeight                "desiredHeight"
+#endif
+#ifndef XmCDesiredHeight
+#define XmCDesiredHeight                "DesiredHeight"
+#endif
+
+#ifndef XmNdesiredWidth
+#define XmNdesiredWidth                 "desiredWidth"
+#endif
+#ifndef XmCDesiredWidth
+#define XmCDesiredWidth                 "DesiredWidth"
+#endif
+
+/* Constraint resources */
+#ifndef XmNattachRow
+#define XmNattachRow "attachRow"
+#endif
+#ifndef XmNattachColumn
+#define XmNattachColumn "attachColumn"
+#endif
+
 #ifndef XbaeIsXbaeMatrix
 #define XbaeIsXbaeMatrix( w)	XtIsSubclass(w, xbaeMatrixWidgetClass)
 #endif /* XbaeIsXbaeMatrix */
 
 /* Class record constants */
 
-externalref WidgetClass xbaeMatrixWidgetClass;
+#ifdef	WIN32
+#define	INTERNALREF	externalref __declspec(dllimport)
+#else
+#define	INTERNALREF	extern
+#endif
+
+INTERNALREF WidgetClass xbaeMatrixWidgetClass;
+
+#undef	INTERNALREF
 
 typedef struct _XbaeMatrixClassRec *XbaeMatrixWidgetClass;
 typedef struct _XbaeMatrixRec *XbaeMatrixWidget;
@@ -596,6 +721,9 @@ extern void XbaeMatrixAddColumns(Widget, int, String *, String *, short *,
 				     Pixel *, int);
 extern void XbaeMatrixAddRows(Widget,  int , String *, String *,
 				  Pixel *, int);
+extern void XbaeMatrixAddVarRows(Widget, int, String *, String *, short *,
+				 int *, unsigned char *, unsigned char *,
+				 Pixel *, int);
 extern void XbaeMatrixCancelEdit(Widget, Boolean);
 extern Boolean XbaeMatrixCommitEdit(Widget, Boolean);
 extern void XbaeMatrixDeleteColumns(Widget, int, int);
@@ -604,12 +732,18 @@ extern void XbaeMatrixDeselectAll(Widget);
 extern void XbaeMatrixDeselectCell(Widget, int, int);
 extern void XbaeMatrixDeselectColumn(Widget, int);
 extern void XbaeMatrixDeselectRow(Widget, int);
+extern void XbaeMatrixDeunderlineCell(Widget, int, int);
+extern void XbaeMatrixDeunderlineColumn(Widget, int);
+extern void XbaeMatrixDeunderlineRow(Widget, int);
 extern void XbaeMatrixEditCell(Widget, int, int);
 extern void XbaeMatrixFirstSelectedCell(Widget, int *, int *);
 extern int XbaeMatrixFirstSelectedColumn(Widget);
 extern int XbaeMatrixFirstSelectedRow(Widget);
 extern String XbaeMatrixGetCell(Widget, int, int);
+extern Pixel XbaeMatrixGetCellBackground(Widget w, int row, int column);
+extern Pixel XbaeMatrixGetCellColor(Widget w, int row, int column);
 extern XtPointer XbaeMatrixGetCellUserData(Widget, int, int);
+extern Widget XbaeMatrixGetCellWidget(Widget, int, int);
 extern XtPointer XbaeMatrixGetColumnUserData(Widget, int);
 extern void XbaeMatrixGetCurrentCell(Widget, int *, int *);
 extern int XbaeMatrixGetEventRowColumn(Widget, XEvent *, int *, int *);
@@ -632,6 +766,9 @@ extern void XbaeMatrixHighlightCell(Widget, int, int);
 extern void XbaeMatrixHighlightRow(Widget, int);
 extern void XbaeMatrixHighlightColumn(Widget, int);
 extern void XbaeMatrixUnhighlightCell(Widget, int, int);
+extern void XbaeMatrixUnderlineCell(Widget, int, int);
+extern void XbaeMatrixUnderlineColumn(Widget, int);
+extern void XbaeMatrixUnderlineRow(Widget, int);
 extern void XbaeMatrixUnhighlightRow(Widget, int);
 extern void XbaeMatrixUnhighlightColumn(Widget, int);
 extern void XbaeMatrixUnhighlightAll(Widget);
@@ -640,7 +777,6 @@ extern void XbaeMatrixSetCellBackground(Widget, int, int, Pixel);
 extern void XbaeMatrixSetCellColor(Widget, int, int, Pixel);
 extern void XbaeMatrixSetCellUserData(Widget, int, int, XtPointer);
 extern void XbaeMatrixSetCellWidget(Widget, int, int, Widget);
-extern Widget XbaeMatrixGetCellWidget(Widget, int, int);
 extern void XbaeMatrixSetColumnBackgrounds(Widget, int, Pixel *, int);
 extern void XbaeMatrixSetColumnColors(Widget, int, Pixel *, int);
 extern void XbaeMatrixSetColumnUserData(Widget, int, XtPointer);
@@ -663,12 +799,27 @@ extern String XbaeMatrixGetRowLabel(Widget, int);
 extern void XbaeMatrixSetColumnLabel(Widget, int, String);
 extern void XbaeMatrixSetRowLabel(Widget, int, String);
 extern Widget XbaeCreateMatrix(Widget, String, ArgList, Cardinal);
+extern void XbaeMatrixSetCurrentCellPosition(Widget w, int current_row, int current_column);
+
+void XbaeMatrixSetCellShadow(Widget w, int row, int column, unsigned char shadow_type);
+void XbaeMatrixSetRowShadow(Widget w, int row, unsigned char shadow_type);
+void XbaeMatrixSetColumnShadow(Widget w, int column, unsigned char shadow_type);
 
 extern void XbaeMatrixSetColumnWidth(Widget mw, int column, int width);
 extern int XbaeMatrixGetColumnWidth(Widget w, int column);
 extern void XbaeMatrixSetRowHeight(Widget w, int row, int height);
 extern int XbaeMatrixGetRowHeight(Widget w, int row);
 
+extern void XbaeMatrixShowColumnArrows(Widget, int, Boolean);
+
+void XbaeMatrixSetCellTag(Widget w, int row, int column, XmStringTag tag);
+
+extern void XbaeMatrixSortRows(Widget w, int (*proc)(Widget, int, int, void *), void *user_data);
+extern void XbaeMatrixSortColumns(Widget w, int (*proc)(Widget, int, int, void *), void *user_data);
+/* ARCAD SYSTEMHAUS */
+extern void XbaeMatrixSetCellPixmap(Widget w, int row, int column, Pixmap pixmap, Pixmap mask);
+extern int XbaeMatrixGetCellPixmap(Widget w, int row, int column, Pixmap * pixmap, Pixmap * mask);
+
 
 typedef unsigned char	Alignment;
 typedef Alignment *	AlignmentArray;
@@ -681,7 +832,7 @@ typedef MaxLength *	MaxLengthArray;
 /*
  * cell shadow types
  */
- 
+
 enum
 {
     XmGRID_NONE		    = 0x00,
@@ -726,11 +877,8 @@ enum {
     HighlightCell	= 0x0001,
     HighlightRow	= 0x0002,
     HighlightColumn	= 0x0004,
-    HighlightOther	= 0x0008,
-    UnhighlightCell	= 0x0010,
-    UnhighlightRow	= 0x0020,
-    UnhighlightColumn	= 0x0040,
-    UnhighlightAll	= UnhighlightCell | UnhighlightRow | UnhighlightColumn
+    /* Deprecated */
+    HighlightOther =  0x0008
 };
 
 /*
@@ -751,7 +899,8 @@ typedef enum _XbaeReasonType
     XbaeProcessDragReason,
     XbaeLabelActivateReason,
     XbaeValueChangedReason,
-    XbaeResizeRowReason
+    XbaeResizeRowReason,
+    XbaeTrackCellReason
 }
 XbaeReasonType;
 
@@ -761,10 +910,21 @@ XbaeReasonType;
 typedef enum
 {
     XbaeString=1,
-    XbaePixmap
+    XbaePixmap=2,
+    XbaeStringFree=5
 }
 XbaeCellType;
 
+/*
+ * Wrap types
+ */
+enum
+{
+    XbaeWrapNone, 
+    XbaeWrapContinuous, 
+    XbaeWrapWord
+};
+
 /*
  * The 'Any' struct which can be used in callbacks used with different
  * Callback structs but only need to access its 4 members
@@ -777,7 +937,7 @@ typedef struct _XbaeMatrixAnyCallbackStruct
     int column;
 }
 XbaeMatrixAnyCallbackStruct;
-    
+
 /*
  * Struct passed to modifyVerifyCallback
  */
@@ -1003,6 +1163,25 @@ typedef struct _XbaeMatrixLabelActivateCallbackStruct
 XbaeMatrixLabelActivateCallbackStruct;
 
 
+typedef struct _XbaeMatrixTrackCellCallbackStruct {
+	XbaeReasonType	reason;
+	XEvent		*event;
+	int		row, column;
+	int		prev_row, prev_column;
+	Position	pointer_x, pointer_y;
+}
+XbaeMatrixTrackCellCallbackStruct;
+
+#if 0
+#define	XtMalloc(x)	dmalloc_malloc(__FILE__, __LINE__, x, 10, 0, 1)
+#define	XtRealloc(x,y)	dmalloc_malloc(__FILE__, __LINE__, x, 12, 0, 1)
+#endif
+
+/* Lesstiff 0.94.0 needs this */
+#ifndef _MOTIF_DEFAULT_LOCALE
+#define _MOTIF_DEFAULT_LOCALE "_MOTIF_DEFAULT_LOCALE"
+#endif
+
 #ifdef __cplusplus
 }
 #endif
